#import "ASMModelParams.h"
#import "Interval.h"
#import <objectbase.h>  //for probes


@implementation ASMModelParams
/*"The ASMModelParams class is a "holding class" for the parameters
  that are associated with the ASMModelSwarm class and it also
  controls the GUI probes that users can use to change variables at
  runtime.  These values are set here in order to keep the code clean
  and neat!  Several parts of the simulation need to have access to
  the information held by ASMModelParams, not just ASMModelParams, but
  also any classes that want information on system parameters.

  A big reason for keeping these values in a separate class is that
  they can be used by both batch and graphical runs of the model.

  There are values saved for these parameters in the asm.scm file, and
  the Parameters class, which orchestrates all this parameter magic,
  does the work of creating one of these ASMModelParams objects."*/

+createBegin: (id) aZone
{
  ASMModelParams * obj;
  id <ProbeMap> probeMap;

  obj = [super createBegin: aZone];

  obj->initholding = [Interval create];
  obj->initialcash = [Interval create];
  obj->minholding = [Interval create];
  obj->mincash = [Interval create];
  obj->intrate = [Interval create];
  obj->baseline = [Interval create];
  obj->mindividend = [Interval create];
  obj->maxdividend = [Interval create];
  obj->amplitude = [Interval create];
  obj->maxprice = [Interval create];
  obj->minprice = [Interval create];
  obj->taup = [Interval create];
  obj->minexcess = [Interval create];
  obj->eta = [Interval create];
  obj->etamax = [Interval create];
  obj->etamin = [Interval create];
  obj->rea = [Interval create];
  obj->reb = [Interval create];
  obj->tauv = [Interval create];
  obj->lambda = [Interval create];
  obj->maxbid = [Interval create];
  obj->initvar = [Interval create];
  obj->maxdev = [Interval create];

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];
  
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numBFagents"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "initholding"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "initialcash"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "minholding"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "mincash"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "intrate"
  			            inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "baseline"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "mindividend"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "maxdividend"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "amplitude"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "period"
  			            inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "maxprice"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "minprice"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "taup"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "exponentialMAs" 
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "sptype"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "maxiterations" 
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "minexcess"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "eta"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "etamin"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "etamax"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "rea"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "reb"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "randomSeed"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "dividendSeed"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "tauv"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "lambda"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "maxbid"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "initvar"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "maxdev"
  			            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "setOutputForData"
  			            inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;

}

-init {
  [initholding set: [Interval constant: d_initholding]];
  [initialcash set: [Interval constant: d_initialcash]];
  [minholding set: [Interval constant: d_minholding]];
  [mincash set: [Interval constant: d_mincash]];
  [intrate set: [Interval constant: d_intrate]];
  [baseline set: [Interval constant: d_baseline]];   //Also used by World.
  [mindividend set: [Interval constant: d_mindividend]];
  [maxdividend set: [Interval constant: d_maxdividend]];
  [amplitude set: [Interval constant: d_amplitude]];
  [maxprice set: [Interval constant: d_maxprice]];
  [minprice set: [Interval constant: d_minprice]];
  [taup set: [Interval constant: d_taup]];
  [minexcess set: [Interval constant: d_minexcess]];
  [eta set: [Interval constant: d_eta]];
  [etamax set: [Interval constant: d_etamax]];
  [etamin set: [Interval constant: d_etamin]];
  [rea set: [Interval constant: d_rea]];
  [reb set: [Interval constant: d_reb]];
  [tauv set: [Interval constant: d_tauv]];          
  [lambda set: [Interval constant: d_lambda]];
  [maxbid set: [Interval constant: d_maxbid]]; 
  [initvar set: [Interval constant: d_initvar]];
  [maxdev set: [Interval constant: d_maxdev]];
  return self;
}


-(void)drop {
  [initholding drop];
  [initialcash drop];
  [minholding drop];
  [mincash drop];
  [intrate drop];
  [baseline drop];
  [mindividend drop];
  [maxdividend drop];
  [amplitude drop];
  [maxprice drop];
  [minprice drop];
  [taup drop];
  [minexcess drop];
  [eta drop];
  [etamax drop];
  [etamin drop];
  [rea drop];
  [reb drop];
  [tauv drop];
  [lambda drop];
  [maxbid drop];
  [initvar drop];
  [maxdev drop];
}

@end
