#import <objectbase/SwarmObject.h>
#import "BitVector.h"

@class Interval;

@interface BFCast: SwarmObject
{
  Interval *forecast;	/*" this forecast of return"*/
  Interval *lforecast;	/*" previous forecast"*/
  Interval *variance;	/*" variance of this forecast"*/
  Interval *strength;      /*"strength=maxdev - variance +specfactor. This was the original sfsm specification, not the ASM-2.0 specification"*/
  Interval *a;		/*" (price + dividend) coefficient"*/
  Interval *b;		/*" dividend coefficient "*/
  Interval *c;		/*" constant term"*/
  Interval *specfactor;	/*" specificty=(condbits - nnulls - specificity)* bitcost. "*/
  Interval *bitcost; /*" cost of using bits in forecasts"*/

  BitVector *conditions; /*" a BitVector object that holds records on which conditions in the world are being monitored by this forecast object"*/
  int lastactive;  /*" last time period in which this forecast was active"*/
  int specificity; /*" specificity "*/
  int count; /*" number of times this forecast has been active"*/
  int condwords; /*"number of words of memory needed to hold the conditions"*/
  int condbits; /*"number of bits of information monitored by this forecast"*/
  int nnulls; /*"number of 'unused' bits that remain in the allocated vector after the 'condbits' bits have been used"*/
};

+createBegin: zone;
- createEnd;

- (void)incrSpecificity;

- (void)decrSpecificity;

- (void)setSpecificity: (int)specificity;

- (int)getSpecificity;

- (void)setConditions: (int *) x;

- (int *)getConditions;

- (BitVector *)getConditionsObject;

- (void)setNNulls: (int)x;

- (void)setBitcost: (Interval *)x;

- (void)setConditionsWord: (int)i To: (int)value;

- (int)getConditionsWord: (int)x;

- (void)setConditionsbit: (int)bit To: (int)x; //works for 0,1,2

- (void)setConditionsbit: (int)bit FromZeroTo: (int)x;//faster if cond[bit]=0

- (void)maskConditionsbit: (int)bit;

- (void)switchConditionsbit: (int)bit;

- (int) getConditionsbit: (int)bit;

- (void) setAval: (Interval *)x;

- (void) setBval: (Interval *)x;

- (void) setCval: (Interval *)x;

- (Interval *)getAval;

- (Interval *)getBval;

- (Interval *)getCval;

- (void)updateSpecfactor;

- (void)setSpecfactor: (Interval *)x;

- (Interval *)getSpecfactor;

- (void)setVariance: (Interval *) x;
       
- (Interval *)getBFCastVariance;
				// Renamed to avoid collision with 
				// -(double)getVariance in Swarm's
				// random/distribution.h
- (void)setCondwords: (int)x;

- (void)setCondbits: (int)x;

- (void)setForecast: (Interval *)x;

- (Interval *)getForecast;

- (Interval *)updateForecastPrice: (Interval *)price Dividend: (Interval *)dividend;

- (void)setLforecast: (Interval *)x;

- (Interval *)getLforecast;

- (void)setLastactive: (int)x;

- (int)getLastactive;

- (void)setCnt: (int)x;

- (int)getCnt;

- (int)incrCount;

- (void)setStrength: (Interval *)x;

- (Interval *)getStrength;

- copyEverythingFrom: (BFCast *)from;

- print;

- printcond: (int)conditions;

- (void)drop;

@end







