#import "MovingAverage.h"
#include <math.h>
#import "Interval.h"

@implementation MovingAverage
/*"This is a general purpose class for creating Moving Averages, either flat "equally weighted" moving averages or exponentially weighted moving averages"*/

+createBegin: zone {
  MovingAverage *obj = [super createBegin: zone];

  obj->sumOfInputs = [Interval create];
  obj->uncorrectedSum = [Interval create];
  obj->expWMA = [Interval create];
  obj->aweight = [Interval create];
  obj->bweight = [Interval create];

  return obj;
}

+create: zone {
  MovingAverage *obj = [self createBegin: zone];

  obj = [obj createEnd];

  return obj;
}


- initWidth: (int)w
{
  int i;
  width=w;
  maInputs=[[self getZone] allocBlock: w * sizeof(Interval *) ];
  for(i=0; i < w; i++)
    {
      maInputs[i] = [Interval createWith: 0.0];
    }
  numInputs = 0;
  [sumOfInputs set: [Interval constant: 0.0]];
  arrayPosition=0;
  [uncorrectedSum set: [Interval constant: 0.0]];
 
  [bweight set: [[[[Interval constant: -1.0]
		    quot: [Interval castInt: w]] expm1] neg]];
				//weight for expWMA
  [aweight set: [[Interval constant: 1.0] diff: bweight]];
				//weight for expWMA; ma=a*ma(x)+b*x

  return self;			   
}



- initWidth: (int)w Value: (Interval *)val
{
  int i;
  width=w;
  maInputs=[[self getZone] allocBlock: w * sizeof(Interval *) ];
  for(i=0; i < w; i++)
    {
      maInputs[i] = [[Interval create] set: val];
    }
  numInputs = w;
  [sumOfInputs set: [[Interval castInt: w] prod: val]];
  arrayPosition=0;
  [uncorrectedSum set: [[Interval castInt: w] prod: val]];
 
  [bweight set: [[[[Interval constant: -1.0]
		    quot: [Interval castInt: w]] expm1] neg]];
				//weight for expWMA
  [aweight set: [[Interval constant: 1.0] diff: bweight]];
				//weight for expWMA; ma=a*ma(x)+b*x
  [expWMA set: val];

  [val drop];

  return self;			   
}


- (int)getNumInputs
{
  return numInputs;
}


- (Interval *)getMA
{
  Interval *movingAverage = [Interval create];
  if (numInputs == 0) [movingAverage set: [Interval constant: 0.0]];

  else if (numInputs < width)
    {
      [movingAverage set: [sumOfInputs quot: [Interval castInt: numInputs]]];
    }
  else
    {
      [movingAverage set: [sumOfInputs quot: [Interval castInt: width]]];
    }
  //printf( "Object: ma=%f  sumOfInput: %f numInputs: %d \n",movingAverage, sumOfInputs, numInputs);
  return [movingAverage ret];
}

- (Interval *)getAverage
{
  if (numInputs ==0) return [[Interval constant: 0.0] ret];
  else return [[uncorrectedSum quot: [Interval castInt: numInputs]] ret];
}

-(Interval *) getEWMA
{
  return [[expWMA clone] ret]; 	
}

- (void)addValue: (Interval *)x;
{
  arrayPosition = (width + numInputs) % width;

  if(numInputs < width)
    {
    [sumOfInputs add: x];
    [maInputs[arrayPosition] set: x];
    }
  else
    {
      [sumOfInputs set: [sumOfInputs diff: [maInputs[arrayPosition] sum: x]]] ;
      [maInputs[arrayPosition] set: x];
    }
  numInputs++;

  [uncorrectedSum add: x];

  [expWMA set: [[aweight prod: expWMA] sum: [bweight prod: x]]];
  [x drop];
}


- (void)drop
{
  int i;

  for(i = 0; i < width; i++) [maInputs[i] drop];
  [sumOfInputs drop];
  [uncorrectedSum drop];
  [expWMA drop];
  [aweight drop];
  [bweight drop];
  [[self getZone] freeBlock: maInputs  blockSize: width*sizeof(Interval *)];
  [super drop];
}



@end


